/**
 * By Gustav Hartvigsson
 * This library provides an abstract class for creating
 * processing like programes, with little hassle and grief.
 */


using SDL;
using SDLGraphics;
using SDLImage;
using SDLMixer;
using SDLTTF;

namespace vProcessing{
  
  
  
  /**
   * This class is the class that you have to enherrent from to get the
   * functinality of the library.
   */
  public abstract class PMain {
    
    
    private bool isRun;
    private int height = 200;
    private int width = 200;
    private unowned SDL.Screen screen;
    private uint32 myColor;
    private bool eventBool[322];
    private SDL.Rect myFillRect;
    
    private mouseEvents myMouseEvents;
    
    /**
     * This is where you set all the attributes and add all the valiables that
     * are needed for the programe to run.
     *
     * use:
     * {{{
     *  public override void setup () {
     *    //code here
     *  }
      * }}}
     */
    public abstract void setup();
    
    
    
    
    /**
     * This is where you put the stuff that is goung to be drawn..
     *
     * use:
     * {{{
     *  public override void setup () {
     *    //code here
     *  }
      * }}}
     */
    public abstract void draw();
    
    /**
     * lead an image from file.
     * 
     * Unlike in processing (java) loadImage is static, so it can be used
     * in classes that do not inherret from PApplet.
     */
    public static SDL.Surface loadImage(string s) {
      SDL.Surface tmpImg = SDLImage.load(s);
      return tmpImg;
    }
    
    
    private void initScreen(){
      uint32 VideoFlags = SurfaceFlag.DOUBLEBUF
                            | SurfaceFlag.HWACCEL
                            | SurfaceFlag.HWSURFACE;
      this.screen = Screen.set_video_mode (this.width, this.height,
                                            32, VideoFlags);
      myFillRect.x = 0;
      myFillRect.y = 0;
      myFillRect.h = (int16)this.height;
      myFillRect.w = (int16)this.width;
      if(this.screen == null){
        stderr.printf("Could not initize video \n");
      }
      SDL.WindowManager.set_caption("vProcessing","");
    } //initScreen
    
    /**
     * This sets the size of the screen.
     *
     * put this in the contructior of the final calss to specify a size
     * of the window
     */
    public void size(int x, int y){
      this.height = x;
      this.width = y;
      
    }
    
    public void DrawRect (int16 x, int16 y, int16 x2, int16 y2) {
      SDLGraphics.Rectangle.fill_color(screen, x, y, x2, y2, myColor);
    }
    
    public void DrawCircle (int16 x, int16 y, int16 radius) {
      SDLGraphics.Circle.fill_color(screen, x, y, radius, myColor);
    }
    
    public void DrawLine (int16 x, int16 y, int16 x2, int16 y2) {
      SDLGraphics.Line.color(screen, x, y, x2, y2, myColor);
    }
    
    /**
     * This is the runner for the pragrame, it will run all the things in setup()
     * and continuasly run draw().
     */
    public void run() {
      this.setup();
      this.isRun = true;
      this.myColor = this.color_from_rgba(255, 255, 255, 255);
      this.initScreen();
      while(isRun) {
        this.process_events();
        this.draw();
        this.screen.flip();
      }
      SDL.quit();
    }
    
    
    public void clear(){
      screen.fill(myFillRect, 0x0000000);
    }
    
    public void setColor(uchar r, uchar g, uchar b, uchar a){
      this.myColor = this.color_from_rgba(r, g, b, a);
    }
    
    public uint32 getColor() {
      return myColor;
    }
    
    /**
     * This function stops the loop..
     */
    public void stop() {
      this.isRun = false;
      stdout.printf("stopping \n");
    }
    
    
    /**
     * This function return true if the mouse in down.
     */
    public bool MouseDown () {
      return myMouseEvents.down;
    }
    
    /**
     * This function returns a mouseEvent structure.
     * 
     * Example:
     * {{{
     * int i = this.getMouseEvent().x;
     * }}}
     * 
     * or:
     * {{{
     * mouseEvents mEvents = this.getMouseEvent()
     * }}}
     */
    public mouseEvents getMouseEvent () {
      return myMouseEvents;
    }
    
    private void process_events(){
      Event event = Event();
      Event.poll(out event);
      switch(event.type){
        case EventType.QUIT:
          this.stop();
          break;
        case EventType.MOUSEMOTION:
          /*stdout.printf("dX: %d, dy: %d, X: %d, Y: %d \n",
                       event.motion.xrel, event.motion.yrel,
                       event.motion.x, event.motion.y);*/
          myMouseEvents.dx = event.motion.xrel;
          myMouseEvents.dy = event.motion.yrel;
          myMouseEvents.y = event.motion.y;
          myMouseEvents.x = event.motion.x;
          break;
        case EventType.MOUSEBUTTONDOWN:
          /*stdout.printf("clicked\n");*/
          myMouseEvents.down = true;
          break;
        case EventType.MOUSEBUTTONUP:
          myMouseEvents.down = false;
          break;
        case EventType.KEYDOWN:
          stdout.printf("keydown\n");
          this.eventBool[event.key.keysym.sym] = true;
          break;
        case EventType.KEYUP:
          stdout.printf("keyup\n");
          this.eventBool[event.key.keysym.sym] = false;
          break;
      }
    } // process_events
    
    /**
     * This is copied from:
     * https://github.com/KlaasHub/Vala-Game-Of-Life/blob/master/SDLGUI.vala
     *
     * Thereby I can not take credit for it...
     */
    private static uint32 color_from_rgba(uint8 r, uint8 g, uint8 b, uint8 a) {
      return r << 24 | g << 16 | b << 8 | a;
    }
    
    /**
     * This returns an array of bools that con be used to prosses stuff...
     */
    public bool[] getKeyEvent() {
      return this.eventBool;
    }
    
  }
  
}//End of namespace
